/*
  Introduction to PETSc vector objects
  Concepts: 
    - Vector creation
    - Values insertion
    - Viewing objects
    - Object destruction
*/

static char help[] = "Builds a parallel vector with 1 component on the first processor, 2 on the second, etc.\n\
  Then each processor adds one to all elements except the last rank.\n\n";

/*
  Include "petscvec.h" so that we can use vectors.
  Note that this file automatically includes:
     petscsys.h    - base PETSc routines
     petscis.h     - index sets
     petscviewer.h - viewers
*/
#include <petscvec.h>

#undef __FUNCT__
#define __FUNCT__ "main"
int main(int argc,char **argv)
{
  PetscErrorCode ierr;
  PetscMPIInt    rank;
  PetscInt       i,N;
  PetscScalar    val = 1.0;
  Vec            x;
  InsertMode     insert_mode;

  PetscInitialize(&argc,&argv,"./petscrc_vecexe",help);
  ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRQ(ierr);

  /*
     Create a parallel vector.
      - In this case, we specify the size of each processor's local
        portion, and PETSc computes the global size.  Alternatively,
        if we pass the global size and use PETSC_DECIDE for the
        local size PETSc will choose a reasonable partition trying
        to put nearly an equal number of elements on each processor.
  */
  ierr = VecCreate(PETSC_COMM_WORLD,&x);CHKERRQ(ierr);
  ierr = VecSetSizes(x,rank+1,PETSC_DECIDE);CHKERRQ(ierr);
  ierr = VecSetType(x,VECSTANDARD);CHKERRQ(ierr);
  ierr = VecSetFromOptions(x);CHKERRQ(ierr);
  ierr = VecGetSize(x,&N);CHKERRQ(ierr);
  ierr = VecSet(x,val);CHKERRQ(ierr);

  /*
     Set the vector elements.
      - If VecSetValues is used, always specify global locations of vector entries.
        Alternatively, you can use VecSetValuesLocal and pass local locations
      - Each processor can contribute any vector entries,
        regardless of which processor "owns" them; any nonlocal
        contributions will be transferred to the appropriate processor
        during the assembly process.
      - In this example, the flag insert_mode = ADD_VALUES indicates that all
        contributions will be added together.
  */
  insert_mode = ADD_VALUES;
  /*************************** EXERCISE ***********************************/
  /*
     Uncomment the next two lines.
     Run different times with same number of procs and see what happens
     -> Vector content changes among different runs. Why?
  */
  /************************************************************************/
  //val = (PetscScalar)rank;
  //insert_mode = INSERT_VALUES;
  for (i=0; i<N-rank; i++) {
    ierr = VecSetValues(x,1,&i,&val,insert_mode);CHKERRQ(ierr);
  }

  /*
     Assemble vector, using the 2-step process:
       VecAssemblyBegin(), VecAssemblyEnd()
     Computations can be done while messages are in transition
     by placing code between these two statements.
  */
  ierr = VecAssemblyBegin(x);CHKERRQ(ierr);
  ierr = VecAssemblyEnd(x);CHKERRQ(ierr);

  /* View the vector */
  ierr = VecView(x,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
  /*************************** EXERCISE ***********************************/
  /*
     Comment VecDestroy call. What happens?
       -> memory leak
  */
  /************************************************************************/
  /* Destroy the vector. All PETSc objects should be destroyed when they are no longer needed */
  ierr = VecDestroy(&x);CHKERRQ(ierr);

  ierr = PetscFinalize();
  return 0;
}

