
!  Introduction to PETSc Vec objects
!  Concepts: 
!    - Vector creation
!    - Vector assembling
!    - Viewing objects
!    - Object destruction

!
!
!  Description: Builds a parallel vector with 1 component on the first
!               processor, 2 on the second, etc.  Then each processor adds
!               one to all elements except the last rank.
!
! -----------------------------------------------------------------------

      program main
      implicit none

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!                    Include files
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
!  The following include statements are required for Fortran programs
!  that use PETSc vectors:
!     petscsys.h    - base PETSc routines
!     petscvec.h    - vectors
!
#include <finclude/petscsys.h>
#include <finclude/petscvec.h>

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!                 Beginning of program
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      Vec     x
      PetscInt N,i,ione
      PetscErrorCode ierr
      PetscMPIInt rank
      PetscScalar  val
      InsertMode insert_mode

      call PetscInitialize('./petscrc_vecexe',ierr)
      CHKERRQ(ierr)
      val   = 1.0
      call MPI_Comm_rank(PETSC_COMM_WORLD,rank,ierr)
      CHKERRQ(ierr)

!  Create a parallel vector.
!   - In this case, we specify the size of the local portion on
!     each processor, and PETSc computes the global size.  Alternatively,
!     if we pass the global size and use PETSC_DECIDE for the
!     local size PETSc will choose a reasonable partition trying
!     to put nearly an equal number of elements on each processor.

      N = rank + 1
      call VecCreate(PETSC_COMM_WORLD,x,ierr)
      CHKERRQ(ierr)
      call VecSetSizes(x,N,PETSC_DECIDE,ierr)
      CHKERRQ(ierr)
      call VecSetType(x,VECSTANDARD,ierr)
      CHKERRQ(ierr)
      call VecSetFromOptions(x,ierr)
      CHKERRQ(ierr)
      call VecGetSize(x,N,ierr)
      CHKERRQ(ierr)
      call VecSet(x,val,ierr)
      CHKERRQ(ierr)

!  Set the vector elements.
!   - Note that VecSetValues() uses 0-based row and column numbers
!     in Fortran as well as in C.
!   - Always specify global locations of vector entries.
!   - Each processor can contribute any vector entries,
!     regardless of which processor "owns" them; any nonlocal
!     contributions will be transferred to the appropriate processor
!     during the assembly process.
!   - In this example, the flag insert_mode = ADD_VALUES indicates that all
!     contributions will be added together.

      insert_mode = ADD_VALUES
! ************************ EXERCISE ********************************
!     Uncomment the next two lines. Run different times with same number of procs and see what happens
!       -> Vector content changes among different runs. Why?
! ******************************************************************
      !val = rank
      !insert_mode = INSERT_VALUES
      ione = 1
      do 100 i=0,N-rank-1
         call VecSetValues(x,ione,i,val,insert_mode,ierr)
         CHKERRQ(ierr)
 100  continue

!  Assemble vector, using the 2-step process:
!    VecAssemblyBegin(), VecAssemblyEnd()
!  Computations can be done while messages are in transition
!  by placing code between these two statements.

      call VecAssemblyBegin(x,ierr)
      CHKERRQ(ierr)
      call VecAssemblyEnd(x,ierr)
      CHKERRQ(ierr)

!  View the vector.

      call VecView(x,PETSC_VIEWER_STDOUT_WORLD,ierr)
      CHKERRQ(ierr)
! ************************ EXERCISE ********************************
!     Comment VecDestroy call. What happens?
!       -> memory leak
! ******************************************************************
!     Destroy the vector. All PETSc objects should be destroyed when they are no longer needed
      call VecDestroy(x,ierr)
      CHKERRQ(ierr)

      call PetscFinalize(ierr)
      end

